using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing;
using System.Windows.Controls;
using System.Windows.Forms;
using Infragistics.Win;
using Infragistics.Win.UltraWinEditors;
using Infragistics.Win.UltraWinGrid;

namespace CS.FID.Gbb.Zeus.View.WPF.Controls.Winforms
{
    public class GroupByRowButtonCreationFilter : IUIElementCreationFilter
    {
        // This class is used add the buttons to the header bar for the callaps and expand all as well
        // as lineing up the harazontal scroll bar with the fixed rows

        private int _fixedColumnsOffset;

        private readonly LayoutUltraGrid _grid;

        public GroupByRowButtonCreationFilter(LayoutUltraGrid grid)
        {
            _grid = grid;
        }

        #region IUIElementCreationFilter Members
        public void AfterCreateChildElements(UIElement parent)
        {
            //Debug.WriteLine("AfterCreateChildElements: " + parent);

            foreach (System.Windows.Forms.Control cnt in _grid.Controls)
            {
                DependentTextUIElement UI = _grid.DisplayLayout.UIElement.ElementFromPoint(cnt.Location) as DependentTextUIElement;
                if (cnt.GetType() == typeof(UltraTextEditor) && UI == null)// && !IsScrol)
                {
                    _grid.Controls.Remove(cnt);
                    Debug.WriteLine("Removing control: " + cnt + ": " + cnt.GetHashCode());
                }
            }


            var dependentTextUiElement = parent as DependentTextUIElement;
            if (dependentTextUiElement != null)
            {

                if (parent.Parent is GroupByRowDescriptionUIElement)
                {
                    var groupByRowDescriptionUiElement = parent.Parent as GroupByRowDescriptionUIElement;

                    var ultraGridGroupByRow = groupByRowDescriptionUiElement.GroupByRow;

                    //Debug.WriteLine(">> ultraGridGroupByRow.ChildBands.FirstRow: " + ultraGridGroupByRow.ChildBands.FirstRow);

                    if (ultraGridGroupByRow.ChildBands.FirstRow.GetType() != typeof(UltraGridGroupByRow))
                    //doesn't have child groups, i.e. leaf-level group
                    {
                        Debug.WriteLine(
                            ">>>>  Does not have child groups, i.e. leaf-level group...inserting bulk update editors...");

                        int x = dependentTextUiElement.Rect.X + 80;
                        int y = dependentTextUiElement.Rect.Y;

                        var ultraTextEditor = new UltraTextEditor
                        {
                            Parent = dependentTextUiElement.Control,
                            BorderStyle = UIElementBorderStyle.None,
                            Location = new Point(x, y),
                            Tag = "BidPrice",
                            Width = 75 //get real value from layout
                        };

                        Debug.WriteLine(">>>>>>  x: " + x + ", y: " + y + " now has " + ultraTextEditor + ": " + ultraTextEditor.GetHashCode());

                        ultraTextEditor.KeyDown += EdKeyDown;
                    }
                }
            }



            if (parent is RowSelectorHeaderUIElement)
            {
                if (_grid != null &&
                   _grid.ViewModel != null)
                {
                    IList<string> groupedColumns = _grid.ViewModel.GetGroupedColumns();

                    if (groupedColumns != null &&
                        groupedColumns.Count != 0)
                    {

                        // get the rect of the parent element inside its borders
                        Rectangle parentRect = parent.RectInsideBorders;
                        int width = parent.RectInsideBorders.Width / groupedColumns.Count;


                        for (int i = 0; i < groupedColumns.Count; ++i)
                        {
                            // create a new button element
                            ExpanderHeaderButtonUIElement btn = new ExpanderHeaderButtonUIElement(parent, _grid);
                            btn.Index = i;
                            // set the text of the button to whatever you want
                            btn.Text = (i + 1).ToString();


                            Rectangle rect = parentRect;
                            rect.Offset(i*width, 0);
                            rect.Width = width;
                            btn.Rect = rect;

                            // append the button element to the child elements collection
                            parent.ChildElements.Add(btn);
                        }
                    }
                }
            }
            else if (parent is FixedCellSeparatorUIElement)
            {
                //Gather the X-axis position of the FixedCellSeparator; this will be used for the x position of the scrollBar
                _fixedColumnsOffset = parent.Rect.Location.X;
            }
            else if (parent is GroupByRowUIElement)
            {
                SummaryFooterUIElement child = parent.GetDescendant(typeof(SummaryFooterUIElement)) as SummaryFooterUIElement;
                UltraGridGroupByRow groupByRow = parent.GetContext(typeof(UltraGridGroupByRow)) as UltraGridGroupByRow;

                if (groupByRow != null && groupByRow.IsExpanded)
                {
                    parent.ChildElements.Remove(child);
                }
            }





        }

        void EdKeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Enter)
            {
                var ultraTextEditor = sender as UltraTextEditor;
                if (ultraTextEditor == null || ultraTextEditor.Text == "" || ultraTextEditor.Value == null) return;

                foreach (UltraGridRow it in _grid.Rows)
                {
                    var dependentTextUiElement = _grid.DisplayLayout.UIElement.ElementFromPoint(ultraTextEditor.Location) as DependentTextUIElement;

                    foreach (UltraGridRow ultraGridRow in it.ChildBands[0].Rows.All)
                    {
                        if (ultraGridRow.IsGroupByRow && ultraGridRow.Description == dependentTextUiElement.Text)
                        {
                            foreach (UltraGridRow row in it.ChildBands[0].Rows[ultraGridRow.Index].ChildBands[0].Rows.All)
                            {
                                row.Cells[3].Value = Convert.ToInt32(row.Cells[3].Value) + Convert.ToInt32(ultraTextEditor.Text);
                            }
                        }
                    }
                }

                ultraTextEditor.Text = null;
            }
        }

        public bool BeforeCreateChildElements(UIElement parent)
        {
            if (parent is ColScrollbarUIElement)
            {
                if (_fixedColumnsOffset != -1)
                {
                    //Create a rectangle object to use as the new DrawRect for the ColScrollbarUIElement
                    Rectangle rect = new Rectangle()
                                         {
                                             X = _fixedColumnsOffset,
                                             Y = parent.Rect.Y,
                                             Width = parent.Rect.Width + parent.Rect.X - _fixedColumnsOffset,
                                             Height = parent.Rect.Height,
                                         };
                    parent.Rect = rect;
                }
                
            }

            // returning false allows the grid to position the child elements correctly
            return false;
        }
       #endregion
    }
}